@@ -1,5 +1,6 @@
 /*
- * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2005, 2007, 2009, 2011-2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 1998-2002  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -33,7 +34,7 @@ fromtext_opt(ARGS_FROMTEXT) {
 	 * OPT records do not have a text format.
 	 */
 
-	REQUIRE(type == dns_rdatatype_opt);
+	REQUIRE(type == 41);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -58,7 +59,7 @@ totext_opt(ARGS_TOTEXT) {
 	 * OPT records do not have a text format.
 	 */
 
-	REQUIRE(rdata->type == dns_rdatatype_opt);
+	REQUIRE(rdata->type == 41);
 
 	dns_rdata_toregion(rdata, &r);
 	while (r.length > 0) {
@@ -66,7 +67,7 @@ totext_opt(ARGS_TOTEXT) {
 		isc_region_consume(&r, 2);
 		length = uint16_fromregion(&r);
 		isc_region_consume(&r, 2);
-		snprintf(buf, sizeof(buf), "%u %u", option, length);
+		sprintf(buf, "%u %u", option, length);
 		RETERR(str_totext(buf, target));
 		INSIST(r.length >= length);
 		if (length > 0) {
@@ -100,7 +101,7 @@ fromwire_opt(ARGS_FROMWIRE) {
 	isc_uint16_t length;
 	unsigned int total;
 
-	REQUIRE(type == dns_rdatatype_opt);
+	REQUIRE(type == 41);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -108,8 +109,6 @@ fromwire_opt(ARGS_FROMWIRE) {
 	UNUSED(options);
 
 	isc_buffer_activeregion(source, &sregion);
-	if (sregion.length == 0)
-		return (ISC_R_SUCCESS);
 	total = 0;
 	while (sregion.length != 0) {
 		if (sregion.length < 4)
@@ -129,7 +128,7 @@ fromwire_opt(ARGS_FROMWIRE) {
 			isc_uint8_t addrbytes;
 
 			if (length < 4)
-				return (DNS_R_OPTERR);
+				return (DNS_R_FORMERR);
 			family = uint16_fromregion(&sregion);
 			isc_region_consume(&sregion, 2);
 			addrlen = uint8_fromregion(&sregion);
@@ -139,25 +138,16 @@ fromwire_opt(ARGS_FROMWIRE) {
 			switch (family) {
 			case 1:
 				if (addrlen > 32U || scope > 32U)
-					return (DNS_R_OPTERR);
+					return (DNS_R_FORMERR);
 				break;
 			case 2:
 				if (addrlen > 128U || scope > 128U)
-					return (DNS_R_OPTERR);
+					return (DNS_R_FORMERR);
 				break;
-			default:
-				return (DNS_R_OPTERR);
 			}
 			addrbytes = (addrlen + 7) / 8;
 			if (addrbytes + 4 != length)
-				return (DNS_R_OPTERR);
-
-			if (addrbytes != 0U && (addrlen % 8) != 0) {
-				isc_uint8_t bits = ~0U << (8 - (addrlen % 8));
-				bits &= sregion.base[addrbytes - 1];
-				if (bits != sregion.base[addrbytes - 1])
-					return (DNS_R_OPTERR);
-			}
+				return (DNS_R_FORMERR);
 			isc_region_consume(&sregion, addrbytes);
 			break;
 		}
@@ -166,17 +156,7 @@ fromwire_opt(ARGS_FROMWIRE) {
 			 * Request has zero length.  Response is 32 bits.
 			 */
 			if (length != 0 && length != 4)
-				return (DNS_R_OPTERR);
-			isc_region_consume(&sregion, length);
-			break;
-		case DNS_OPT_COOKIE:
-			if (length != 8 && (length < 16 || length > 40))
-				return (DNS_R_OPTERR);
-			isc_region_consume(&sregion, length);
-			break;
-		case DNS_OPT_KEY_TAG:
-			if (length == 0 || (length % 2) != 0)
-				return (DNS_R_OPTERR);
+				return (DNS_R_FORMERR);
 			isc_region_consume(&sregion, length);
 			break;
 		default:
@@ -200,7 +180,7 @@ fromwire_opt(ARGS_FROMWIRE) {
 static inline isc_result_t
 towire_opt(ARGS_TOWIRE) {
 
-	REQUIRE(rdata->type == dns_rdatatype_opt);
+	REQUIRE(rdata->type == 41);
 
 	UNUSED(cctx);
 
@@ -214,7 +194,7 @@ compare_opt(ARGS_COMPARE) {
 
 	REQUIRE(rdata1->type == rdata2->type);
 	REQUIRE(rdata1->rdclass == rdata2->rdclass);
-	REQUIRE(rdata1->type == dns_rdatatype_opt);
+	REQUIRE(rdata1->type == 41);
 
 	dns_rdata_toregion(rdata1, &r1);
 	dns_rdata_toregion(rdata2, &r2);
@@ -227,7 +207,7 @@ fromstruct_opt(ARGS_FROMSTRUCT) {
 	isc_region_t region;
 	isc_uint16_t length;
 
-	REQUIRE(type == dns_rdatatype_opt);
+	REQUIRE(type == 41);
 	REQUIRE(source != NULL);
 	REQUIRE(opt->common.rdtype == type);
 	REQUIRE(opt->common.rdclass == rdclass);
@@ -257,7 +237,7 @@ tostruct_opt(ARGS_TOSTRUCT) {
 	dns_rdata_opt_t *opt = target;
 	isc_region_t r;
 
-	REQUIRE(rdata->type == dns_rdatatype_opt);
+	REQUIRE(rdata->type == 41);
 	REQUIRE(target != NULL);
 
 	opt->common.rdclass = rdata->rdclass;
@@ -280,7 +260,7 @@ freestruct_opt(ARGS_FREESTRUCT) {
 	dns_rdata_opt_t *opt = source;
 
 	REQUIRE(source != NULL);
-	REQUIRE(opt->common.rdtype == dns_rdatatype_opt);
+	REQUIRE(opt->common.rdtype == 41);
 
 	if (opt->mctx == NULL)
 		return;
@@ -292,7 +272,7 @@ freestruct_opt(ARGS_FREESTRUCT) {
 
 static inline isc_result_t
 additionaldata_opt(ARGS_ADDLDATA) {
-	REQUIRE(rdata->type == dns_rdatatype_opt);
+	REQUIRE(rdata->type == 41);
 
 	UNUSED(rdata);
 	UNUSED(add);
@@ -308,7 +288,7 @@ digest_opt(ARGS_DIGEST) {
 	 * OPT records are not digested.
 	 */
 
-	REQUIRE(rdata->type == dns_rdatatype_opt);
+	REQUIRE(rdata->type == 41);
 
 	UNUSED(rdata);
 	UNUSED(digest);
@@ -320,7 +300,7 @@ digest_opt(ARGS_DIGEST) {
 static inline isc_boolean_t
 checkowner_opt(ARGS_CHECKOWNER) {
 
-	REQUIRE(type == dns_rdatatype_opt);
+	REQUIRE(type == 41);
 
 	UNUSED(type);
 	UNUSED(rdclass);
@@ -332,7 +312,7 @@ checkowner_opt(ARGS_CHECKOWNER) {
 static inline isc_boolean_t
 checknames_opt(ARGS_CHECKNAMES) {
 
-	REQUIRE(rdata->type == dns_rdatatype_opt);
+	REQUIRE(rdata->type == 41);
 
 	UNUSED(rdata);
 	UNUSED(owner);
@@ -350,7 +330,7 @@ isc_result_t
 dns_rdata_opt_first(dns_rdata_opt_t *opt) {
 
 	REQUIRE(opt != NULL);
-	REQUIRE(opt->common.rdtype == dns_rdatatype_opt);
+	REQUIRE(opt->common.rdtype == 41);
 	REQUIRE(opt->options != NULL || opt->length == 0);
 
 	if (opt->length == 0)
@@ -366,7 +346,7 @@ dns_rdata_opt_next(dns_rdata_opt_t *opt) {
 	isc_uint16_t length;
 
 	REQUIRE(opt != NULL);
-	REQUIRE(opt->common.rdtype == dns_rdatatype_opt);
+	REQUIRE(opt->common.rdtype == 41);
 	REQUIRE(opt->options != NULL && opt->length != 0);
 	REQUIRE(opt->offset < opt->length);
 
@@ -387,7 +367,7 @@ dns_rdata_opt_current(dns_rdata_opt_t *opt, dns_rdata_opt_opcode_t *opcode) {
 
 	REQUIRE(opt != NULL);
 	REQUIRE(opcode != NULL);
-	REQUIRE(opt->common.rdtype == dns_rdatatype_opt);
+	REQUIRE(opt->common.rdtype == 41);
 	REQUIRE(opt->options != NULL);
 	REQUIRE(opt->offset < opt->length);
 
