@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011, 2012, 2014-2016  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2011-2015  Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,6 +14,8 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
+/* $Id$ */
+
 /* ! \file */
 
 #include <config.h>
@@ -23,13 +25,12 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <isc/hash.h>
+#include <isc/crc64.h>
 #include <isc/hmacmd5.h>
 #include <isc/hmacsha.h>
 #include <isc/md5.h>
 #include <isc/sha1.h>
 #include <isc/util.h>
-#include <isc/print.h>
 #include <isc/string.h>
 
 /*
@@ -39,7 +40,7 @@
 unsigned char digest[ISC_SHA512_DIGESTLENGTH];
 unsigned char buffer[1024];
 const char *s;
-char str[2 * ISC_SHA512_DIGESTLENGTH + 3];
+char str[2 * ISC_SHA512_DIGESTLENGTH + 1];
 unsigned char key[20];
 
 isc_result_t
@@ -1784,111 +1785,72 @@ ATF_TC_BODY(isc_hmacmd5, tc) {
 	}
 }
 
-ATF_TC(isc_hash_function);
-ATF_TC_HEAD(isc_hash_function, tc) {
-	atf_tc_set_md_var(tc, "descr", "Hash function test");
+/* CRC64 Test */
+ATF_TC(isc_crc64);
+ATF_TC_HEAD(isc_crc64, tc) {
+	atf_tc_set_md_var(tc, "descr", "64-bit cyclic redundancy check");
 }
-ATF_TC_BODY(isc_hash_function, tc) {
-	unsigned int h1;
-	unsigned int h2;
+ATF_TC_BODY(isc_crc64, tc) {
+	isc_uint64_t crc;
+	int i;
 
 	UNUSED(tc);
 
-	/* Incremental hashing */
+	hash_testcase_t testcases[] = {
+		{
+			TEST_INPUT(""),
+			"0x0000000000000000", 1
+		},
+		{
+			TEST_INPUT("a"),
+			"0x9AA9C0AC27F473CE", 1
+		},
+		{
+			TEST_INPUT("abc"),
+			"0x0297F4F93A818B04", 1
+		},
+		{
+			TEST_INPUT("message digest"),
+			"0xF47B357AEAF97352", 1
+		},
+		{
+			TEST_INPUT("abcdefghijklmnopqrstuvwxyz"),
+			"0xA1AA8B21F979F059", 1
+		},
+		{
+			TEST_INPUT("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklm"
+				   "nopqrstuvwxyz0123456789"),
+			"0xFBB6781EF7A86DA3", 1
+		},
+		{
+			TEST_INPUT("123456789012345678901234567890123456789"
+				   "01234567890123456789012345678901234567890"),
+			"0x4A87E7C873EBE581", 1
+		},
+		{ NULL, 0, NULL, 1 }
+	};
 
-	h1 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
-	h1 = isc_hash_function("This ", 5, ISC_TRUE, &h1);
-	h1 = isc_hash_function("is ", 3, ISC_TRUE, &h1);
-	h1 = isc_hash_function("a long test", 12, ISC_TRUE, &h1);
+	hash_testcase_t *testcase = testcases;
 
-	h2 = isc_hash_function("This is a long test", 20,
-			       ISC_TRUE, NULL);
+	while (testcase->input != NULL && testcase->result != NULL) {
+		isc_crc64_init(&crc);
+		for(i = 0; i < testcase->repeats; i++) {
+			isc_crc64_update(&crc,
+				       (const isc_uint8_t *) testcase->input,
+				       testcase->input_len);
+		}
+		isc_crc64_final(&crc);
+		tohexstr((unsigned char *) &crc, sizeof(crc), str);
+		ATF_CHECK_STREQ(str, testcase->result);
 
-	ATF_CHECK_EQ(h1, h2);
-
-	/* Immutability of hash function */
-	h1 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
-	h2 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
-
-	ATF_CHECK_EQ(h1, h2);
-
-	/* Hash function characteristics */
-	h1 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
-	h2 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
-
-	ATF_CHECK_EQ(h1, h2);
-
-	/* Case */
-	h1 = isc_hash_function("Hello world", 12, ISC_FALSE, NULL);
-	h2 = isc_hash_function("heLLo WorLd", 12, ISC_FALSE, NULL);
-
-	ATF_CHECK_EQ(h1, h2);
-
-	/* Unequal */
-	h1 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
-	h2 = isc_hash_function("heLLo WorLd", 12, ISC_TRUE, NULL);
-
-	ATF_CHECK(h1 != h2);
-}
-
-
-ATF_TC(isc_hash_function_reverse);
-ATF_TC_HEAD(isc_hash_function_reverse, tc) {
-	atf_tc_set_md_var(tc, "descr", "Reverse hash function test");
-}
-ATF_TC_BODY(isc_hash_function_reverse, tc) {
-	unsigned int h1;
-	unsigned int h2;
-
-	UNUSED(tc);
-
-	/* Incremental hashing */
-
-	h1 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
-	h1 = isc_hash_function_reverse("\000", 1, ISC_TRUE, &h1);
-	h1 = isc_hash_function_reverse("\003org", 4, ISC_TRUE, &h1);
-	h1 = isc_hash_function_reverse("\007example", 8, ISC_TRUE, &h1);
-
-	h2 = isc_hash_function_reverse("\007example\003org\000", 13,
-				       ISC_TRUE, NULL);
-
-	ATF_CHECK_EQ(h1, h2);
-
-	/* Immutability of hash function */
-	h1 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
-	h2 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
-
-	ATF_CHECK_EQ(h1, h2);
-
-	/* Hash function characteristics */
-	h1 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
-	h2 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
-
-	ATF_CHECK_EQ(h1, h2);
-
-	/* Case */
-	h1 = isc_hash_function_reverse("Hello world", 12, ISC_FALSE, NULL);
-	h2 = isc_hash_function_reverse("heLLo WorLd", 12, ISC_FALSE, NULL);
-
-	ATF_CHECK_EQ(h1, h2);
-
-	/* Unequal */
-	h1 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
-	h2 = isc_hash_function_reverse("heLLo WorLd", 12, ISC_TRUE, NULL);
-
-	ATF_CHECK(h1 != h2);
+		testcase++;
+	}
 }
 
 /*
  * Main
  */
 ATF_TP_ADD_TCS(tp) {
-	/*
-	 * Tests of hash functions, including isc_hash and the
-	 * various cryptographic hashes.
-	 */
-	ATF_TP_ADD_TC(tp, isc_hash_function);
-	ATF_TP_ADD_TC(tp, isc_hash_function_reverse);
 	ATF_TP_ADD_TC(tp, isc_hmacmd5);
 	ATF_TP_ADD_TC(tp, isc_hmacsha1);
 	ATF_TP_ADD_TC(tp, isc_hmacsha224);
@@ -1901,5 +1863,7 @@ ATF_TP_ADD_TCS(tp) {
 	ATF_TP_ADD_TC(tp, isc_sha256);
 	ATF_TP_ADD_TC(tp, isc_sha384);
 	ATF_TP_ADD_TC(tp, isc_sha512);
+	ATF_TP_ADD_TC(tp, isc_crc64);
 	return (atf_no_error());
 }
+
